@@ -67,15 +67,17 @@ digit = re.compile(r'[0-9]')
 
 positionable_operators = frozenset(
     (">>> === !== " +
-    "<< && >= ** != == <= >> || " +
+    "<< && >= ** != == <= >> || |> " +
     "< / - + > : & % ? ^ | *").split(' '))
 
 punct =  (">>>= " +
     "... >>= <<= === >>> !== **= " +
-    "=> ^= :: /= << <= == && -= >= >> != -- += ** || ++ %= &= *= |= " +
+    "=> ^= :: /= << <= == && -= >= >> != -- += ** || ++ %= &= *= |= |> " +
     "= ! ? > < : / ^ - + * & % ~ |")
 
 punct = re.compile(r'([-[\]{}()*+?.,\\^$|#])').sub(r'\\\1', punct)
+# ?. but not if followed by a number
+punct = '\\?\\.(?!\\d) ' + punct
 punct = punct.replace(' ', '|')
 
 punct_pattern = re.compile(punct)
@@ -191,13 +193,13 @@ class Tokenizer(BaseTokenizer):
         if c is None:
             token = self._create_token(TOKEN.EOF, '')
 
+        token = token or self._read_non_javascript(c)
         token = token or self._read_string(c)
         token = token or self._read_word(previous_token)
         token = token or self._read_singles(c)
         token = token or self._read_comment(c)
         token = token or self._read_regexp(c, previous_token)
         token = token or self._read_xml(c, previous_token)
-        token = token or self._read_non_javascript(c)
         token = token or self._read_punctuation()
         token = token or self._create_token(TOKEN.UNKNOWN, self._input.next())
 
@@ -434,6 +436,8 @@ class Tokenizer(BaseTokenizer):
         if resulting_string != '':
             if resulting_string == '=':
                 token = self._create_token(TOKEN.EQUALS, resulting_string)
+            elif resulting_string == '?.':
+                token = self._create_token(TOKEN.DOT, resulting_string)
             else:
                 token = self._create_token(TOKEN.OPERATOR, resulting_string)
 
